#!/bin/ksh

#-------------------------------------------------------------------------
# Special development overrides
#-------------------------------------------------------------------------
developmentOverrideToSkipCdromVerification='no'
bypassSSlevelCheck='no'
devOneToContinueOverride='no'

#-------------------------------------------------------------------------
# Control flags to disable portions of script execution
#-------------------------------------------------------------------------

skipAixInstallCdrom1_aix_base_install='no'
skipAixInstallCdrom1='no'
skipAixInstallCdrom2='no'
skipAixInstallCdrom3='no'
skipAixInstallCdrom4='no'

skipNsmPackageOn3466UpdateCdrom1='no'
skipNsmDocPackageOn3466UpdateCdrom1='no'
skipAixBaseDocOn3466UpdateCdrom1='no'

skipAixMaintenance4334On3466UpdateCdrom2='no'
skipAtapeOn3466UpdateCdrom2='no'
skipAtlddOn3466UpdateCdrom2='no'
skipServiceDirectorOn3466UpdateCdrom2='no'

skipAixMaintenance433xOnAixMaintenanceCdrom1='no'
skipMiscAixMaintenanceOnOnAixMaintenanceCdrom1='no'
skipSsaMaintenanceOnAixMaintenanceCdrom1='no'

skipBringingServerDown='no'
skipUsrFilesystemAdjustment='no'
skipSsadload='no'
skipSsa_sesdld='no'
skipCdFilesystemRemoval='no'

#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
#
# (C) COPYRIGHT International Business Machines Corp. 2000
# All Rights Reserved
# Licensed Materials - Property of IBM
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#@ ---------------------------------------------------------------------
#@
#@ Script Name: h27349a3.ec
#@
#@ Description: This script updates 3466 Cx0 system to AIX 4.3.3.0
#@              as well as other resources 
#@
#@              This script requires that EC D19383 be applied. 
#@              It is the invoker's responsibility to verify this
#@              by invoking the 'lsSSlevel' command.
#@
#@              This script requires the system be a 3466 Cx0. This 
#@              is verified by checking that the system is an 
#@              RS/6000 H50.
#@
#@ Special flags
#@
#@ Example usage:
#@
#@   h27349a3.ec -h    : issue help text 
#@   h27349a3.ec       : run all update phases
#@   h27349a3.ec -b    : bypass check for pre-req EC D19383
#@ 
#@ Log Files:
#@ ----------
#@   Log files are kept in the directory /tmp/updateEc_<script name> Log files for  
#@   an invocation will be prefixed with the same timestamp. The
#@   time stamp for a file is in the format mmdd_hhmm. Specific log files 
#@
#@   All phases
#@   ----------
#@     -- <mmdd_hhmm>.consoleLog - screen output
#@
#@   AIX update phase
#@   ----------------
#@     -- <mmdd_hhmm>.aixdisk1.log - output from update for AIX cdrom #1
#@     -- <mmdd_hhmm>.aixdisk2.log - output from update for AIX cdrom #2
#@     -- <mmdd_hhmm>.aixdisk3.log - output from update for AIX cdrom #3
#@     -- <mmdd_hhmm>.aixdisk4.log - output from update for AIX cdrom #4
#@     -- <mmdd_hhmm>.basedoc.log - output from AIX documentation update
#@     -- <mmdd_hhmm>.fixes.log - output from AIX documentation update
#@
#
# Change History   Date           Description
# --------------   ------------   --------------------------------------
# 10/26/2000       Chris Knapp    Create 
# 02/19/2001       Chris Knapp    Add ssadload
# 04/20/2001       Chris Knapp    .toc checks and new ssa code
# 09.06.2001       Howard Jiang   Added install of 4 additional AIX FC
#                                 filesets
#
#-------------------------------------------------------------------------

enterOneToContinue ()
{
  if [ "$1" = 'always' ]
  then
    devOneToContinueOverride='no'
  fi
  if [ "$devOneToContinueOverride" = 'no' ]
  then
    echo
    echo "Make selection to continue"
    echo
    sleep 1 # seems to be required, or output gets messed up
    PS3=''
    select userResponse in \
        'Enter 1 followed by the ENTER key to continue

             -- OR --
  
         Enter CNTL-C to terminate program'
    do
      case $REPLY in
        1) :
           echo 
           echo 'User entered 1, continuing ...........'
           ;;
        *) echo
           echo "User entered: '$REPLY'. The only valid response is 1"
           echo
           echo 'Enter 1 followed by the ENTER key to continue'
           echo 
           echo '---- OR ----'
           echo 
           echo 'Enter CNTL-C to force program termination'
           echo
           ;;
        esac

      if [[ -n $userResponse ]] ; then
        break
      fi
    done
  fi

} # end enterOneToContinue


# ----------------------------------------------------------------
# CNTL-C trap routine
# ----------------------------------------------------------------
cntlCtrap ()
{
  echo
  echo "User entered CNTL-C"
  echo

  theCommand='umount /cd'
  echo
  echo "Issuing command to unmount /cd: '$theCommand' (note: may not be mounted)"
  eval $theCommand

  theCommand='rmfs /cd'
  echo
  echo "Issuing command to remove /cd filesystem: '$theCommand' (note: may not exist)"
  eval $theCommand

} # end cntlCtrap

initializeInstallationDirectories ()
{
  aix_installation_cdrom1_dir="/cd/usr/sys/inst.images"
  aix_installation_cdrom2_dir="/cd/usr/sys/inst.images"
  aix_installation_cdrom3_dir="/cd/usr/sys/inst.images"
  aix_installation_cdrom4_dir="/cd/usr/sys/inst.images"

  aix_installation_cdrom1_altdir="/storsmart/aixCdroms/aix4330/aixdisk1Dir"
  aix_installation_cdrom2_altdir="/storsmart/aixCdroms/aix4330/aixdisk2Dir"
  aix_installation_cdrom3_altdir="/storsmart/aixCdroms/aix4330/aixdisk3Dir"
  aix_installation_cdrom4_altdir="/storsmart/aixCdroms/aix4330/aixdisk4Dir"

  # -----------------------------------------------------------------------------------------

  nsm_update_cdrom1_altdir='/storsmart/fieldbills/h27349a_cdroms_Cx0_aix4330/BASEDOC_NSM_KIT_1_19P3529.cdrom_1of2_19P3530'
  nsm_update_cdrom1_nsm_package="/cd/nsm.4.1.0.1"
  nsm_update_cdrom1_nsmdoc_package="/cd/nsm.html.3.1.0.33"
  nsm_update_cdrom1_aix_baseDoc="/cd/baseDoc/usr/sys/inst.images"

  # -----------------------------------------------------------------------------------------

  nsm_update_cdrom2_altdir='/storsmart/fieldbills/h27349a_cdroms_Cx0_aix4330/AIX_MAINT_DOC_KIT_1_19P3529.cdrom_2of2_19P3531'
  nsm_update_cdrom2_recommendedMaintenance4334="/cd/usr/sys/inst.images"
  nsm_update_cdrom2_serviceDirector="/cd/serviceDirector.3.0.0.6"
  nsm_update_cdrom2_atldd="/cd/atldd.4.1.5.0"
  nsm_update_cdrom2_Atape="/cd/atape.5.3.9.0"

  # -----------------------------------------------------------------------------------------

  nsm_aix433x_update_cdrom1_altdir='/storsmart/fieldbills/h27349a_cdroms_Cx0_aix4330/AIX_MAINT_4.3.3.x.KIT_2_19P3562.cdrom_1of1_19P3562'
  nsm_aixMaintenance433x_update_cdrom1="/cd/usr/sys/inst.images"
  miscMaintenance_aixMaintenance433x_update_cdrom1="/cd/pieceMealFixes"
  ssa_aixMaintenance433x_update_cdrom1="/cd/ssa"

} # end initializeInstallationDirectories

displayControlVariables ()
{
  # -----------------------------------------------------------
  # display control variables if user has changed any values to skip
  # -----------------------------------------------------------
  if [ "$skipAixInstallCdrom1_aix_base_install" = 'yes' ] || \
     [ "$skipAixInstallCdrom1" = 'yes' ] || \
     [ "$skipAixInstallCdrom2" = 'yes' ] || \
     [ "$skipAixInstallCdrom3" = 'yes' ] || \
     [ "$skipAixInstallCdrom4" = 'yes' ] || \
     [ "$skipNsmPackageOn3466UpdateCdrom1" = 'yes' ] || \
     [ "$skipNsmDocPackageOn3466UpdateCdrom1" = 'yes' ] || \
     [ "$skipAixBaseDocOn3466UpdateCdrom1" = 'yes' ] || \
     [ "$skipAtapeOn3466UpdateCdrom2" = 'yes' ] || \
     [ "$skipAtlddOn3466UpdateCdrom2" = 'yes' ] || \
     [ "$skipServiceDirectorOn3466UpdateCdrom2" = 'yes' ] || \
     [ "$skipAixMaintenance4334On3466UpdateCdrom2" = 'yes' ] || \
     [ "$skipAixMaintenance433xOnAixMaintenanceCdrom1" = 'yes' ] || \
     [ "$skipMiscAixMaintenanceOnOnAixMaintenanceCdrom1" = 'yes' ] || \
     [ "$skipSsaMaintenanceOnAixMaintenanceCdrom1" = 'yes' ] || \
     [ "$skipBringingServerDown" = 'yes' ] || \
     [ "$skipUsrFilesystemAdjustment" = 'yes' ] || \
     [ "$skipCdFilesystemRemoval" = 'yes' ] || \
     [ "$skipSsadload" = 'yes' ] || \
     [ "$skipSsa_sesdld" = 'yes' ] 
  then
    echo 
    echo "********************************************************************************"
    echo '*'
    echo '* User has specified to change one or more control variables'
    echo '*'
    echo "********************************************************************************"
    echo
    echo "skipAixInstallCdrom1_aix_base_install='$skipAixInstallCdrom1_aix_base_install'"
    echo "skipAixInstallCdrom1='$skipAixInstallCdrom1'"
    echo "skipAixInstallCdrom2='$skipAixInstallCdrom2'"
    echo "skipAixInstallCdrom3='$skipAixInstallCdrom3'"
    echo "skipAixInstallCdrom4='$skipAixInstallCdrom4'"
    echo
    echo "skipNsmPackageOn3466UpdateCdrom1='$skipNsmPackageOn3466UpdateCdrom1'"
    echo "skipNsmDocPackageOn3466UpdateCdrom1='$skipNsmDocPackageOn3466UpdateCdrom1'"
    echo "skipAixBaseDocOn3466UpdateCdrom1='$skipAixBaseDocOn3466UpdateCdrom1'"
    echo
    echo "skipAixMaintenance4334On3466UpdateCdrom2='$skipAixMaintenance4334On3466UpdateCdrom2'"
    echo "skipAtapeOn3466UpdateCdrom2='$skipAtapeOn3466UpdateCdrom2'"
    echo "skipAtlddOn3466UpdateCdrom2='$skipAtlddOn3466UpdateCdrom2'"
    echo "skipServiceDirectorOn3466UpdateCdrom2='$skipServiceDirectorOn3466UpdateCdrom2'"
    echo
    echo "skipAixMaintenance433xOnAixMaintenanceCdrom1='$skipAixMaintenance433xOnAixMaintenanceCdrom1'"
    echo "skipMiscAixMaintenanceOnOnAixMaintenanceCdrom1 '$skipMiscAixMaintenanceOnOnAixMaintenanceCdrom1'"
    echo "skipSsaMaintenanceOnAixMaintenanceCdrom1 '$skipSsaMaintenanceOnAixMaintenanceCdrom1'"
    echo
    echo "skipBringingServerDown='$skipBringingServerDown'"
    echo "skipUsrFilesystemAdjustment='$skipUsrFilesystemAdjustment'"
    echo "skipSsadload='$skipSsadload'"
    echo "skipSsa_sesdld='$skipSsa_sesdld'"
    echo "skipCdFilesystemRemoval='$skipCdFilesystemRemoval'"
    echo 
    enterOneToContinue
  fi

} # end displayControlVariables

adjustSizeOfUsrFilesystem ()
{
  echo
  echo "INFO ($scriptName): this routine adjusts the size of the /usr filesystem"
  echo "    to accomodate the upgrades"
  echo
  requiredForUpgrade='447992' # actual amount observed

  currentAllocForUsr=`df /usr | grep -v '512-blocks' | awk '{print $2}'`
  currentFreeInUsr=`df /usr | grep -v '512-blocks' | awk '{print $3}'`
  currentUsedInUsr=$(($currentAllocForUsr - $currentFreeInUsr))

  freePPsOnHdisk0=`lspv hdisk0 | grep 'FREE PPs:' | awk '{print $3}'`
  ppSizeInMbForHdisk0=`lspv hdisk0 | grep 'PP SIZE:' | awk '{print $3}'`
  mbFreeOnHdisk0=$(($freePPsOnHdisk0 * $ppSizeInMbForHdisk0))
  availableOnHdisk0=$(($mbFreeOnHdisk0 * 2048))
  totalAvailableOnHdisk0=$(($currentAllocForUsr + $availableOnHdisk0))

  freePPsOnHdisk1=`lspv hdisk1 | grep 'FREE PPs:' | awk '{print $3}'`
  ppSizeInMbForHdisk1=`lspv hdisk1 | grep 'PP SIZE:' | awk '{print $3}'`
  mbFreeOnHdisk1=$(($freePPsOnHdisk1 * $ppSizeInMbForHdisk1))
  availableOnHdisk1=$(($mbFreeOnHdisk1 * 2048))
  totalAvailableOnHdisk1=$(($currentAllocForUsr + $availableOnHdisk1))

  totalRequiredForUpgradePlus0Percent=$(($currentUsedInUsr + $requiredForUpgrade))
  requiredAfterUpgradePlus1Percent=$(($totalRequiredForUpgradePlus0Percent / 100))

  targetSizeAfterUpgradePlus20Percent=$(($totalRequiredForUpgradePlus0Percent + ($requiredAfterUpgradePlus1Percent * 20)))
  targetSizeAfterUpgradePlus10Percent=$(($totalRequiredForUpgradePlus0Percent + ($requiredAfterUpgradePlus1Percent * 10)))
  targetSizeAfterUpgradePlus5Percent=$(($totalRequiredForUpgradePlus0Percent + ($requiredAfterUpgradePlus1Percent * 5)))
  targetSizeAfterUpgradePlus1Percent=$(($totalRequiredForUpgradePlus0Percent + $requiredAfterUpgradePlus1Percent))

# additionalRequiredForPlus20Percent=$(($targetSizeAfterUpgradePlus20Percent - $currentAllocForUsr - $currentFreeInUsr))
# additionalRequiredForPlus10Percent=$(($targetSizeAfterUpgradePlus10Percent - $currentAllocForUsr - $currentFreeInUsr))
# additionalRequiredForPlus5Percent=$(($targetSizeAfterUpgradePlus5Percent - $currentAllocForUsr - $currentFreeInUsr))
# additionalRequiredForPlus1Percent=$(($targetSizeAfterUpgradePlus1Percent - $currentAllocForUsr - $currentFreeInUsr))

  echo 
  echo "currentAllocForUsr $currentAllocForUsr"
  echo "currentFreeInUsr $currentFreeInUsr"
  echo "currentUsedInUsr $currentUsedInUsr"
  echo
  echo "ppSizeInMbForHdisk0 $ppSizeInMbForHdisk0"
  echo "mbFreeOnHdisk0 $mbFreeOnHdisk0"
  echo "freePPsOnHdisk0 $freePPsOnHdisk0"
  echo "availableOnHdisk0 $availableOnHdisk0"
  echo "totalAvailableOnHdisk0 $totalAvailableOnHdisk0"
  echo
  echo "ppSizeInMbForHdisk1 $ppSizeInMbForHdisk1"
  echo "mbFreeOnHdisk1 $mbFreeOnHdisk1"
  echo "freePPsOnHdisk1 $freePPsOnHdisk1"
  echo "availableOnHdisk1 $availableOnHdisk1"
  echo "totalAvailableOnHdisk1 $totalAvailableOnHdisk1"
  echo 
  echo "requiredForUpgrade $requiredForUpgrade"
  echo "requiredAfterUpgradePlus1Percent $requiredAfterUpgradePlus1Percent"
  echo
  echo "targetSizeAfterUpgradePlus20Percent $targetSizeAfterUpgradePlus20Percent"
  echo "targetSizeAfterUpgradePlus10Percent $targetSizeAfterUpgradePlus10Percent"
  echo "targetSizeAfterUpgradePlus5Percent $targetSizeAfterUpgradePlus5Percent"
  echo "targetSizeAfterUpgradePlus1Percent $targetSizeAfterUpgradePlus1Percent"
  echo "totalRequiredForUpgradePlus0Percent $totalRequiredForUpgradePlus0Percent"

# echo
# echo "additionalRequiredForPlus20Percent $additionalRequiredForPlus20Percent"
# echo "additionalRequiredForPlus10Percent $additionalRequiredForPlus10Percent"
# echo "additionalRequiredForPlus5Percent $additionalRequiredForPlus5Percent"
# echo "additionalRequiredForPlus1Percent $additionalRequiredForPlus1Percent"

  echo
  if (( $currentAllocForUsr >= $targetSizeAfterUpgradePlus10Percent))
  then
    echo "INFO ($scriptName): The number of 512 blocks for the /usr filesystem: $currentAllocForUsr"
    echo " is already sufficient to accomodate the upgrade" 
    theCommand='space currently OK'
  else
    if (( $totalAvailableOnHdisk0 >= $targetSizeAfterUpgradePlus20Percent))
    then
      echo "INFO ($scriptName): ready to increase the size of /usr to the required size + 20% reserve"
      targetSize=$targetSizeAfterUpgradePlus20Percent
      theCommand="chfs -a size=$targetSize /usr"
    else
      echo
      echo "Insufficient space on hdisk0 to increase the size of /usr with 20% reserve"
      echo
      if (( $totalAvailableOnHdisk0 >= $targetSizeAfterUpgradePlus10Percent))
      then
        echo "INFO ($scriptName): ready to increase the size of /usr to the required size + 10% reserve"
        targetSize=$targetSizeAfterUpgradePlus10Percent
        theCommand="chfs -a size=$targetSize /usr"
      else
        echo
        echo "Insufficient space on hdisk0 to increase the size of /usr with 10% reserve"
        echo
        if (( $totalAvailableOnHdisk0 >= $targetSizeAfterUpgradePlus5Percent))
        then
          echo "INFO ($scriptName): ready to increase the size of /usr to the required size + 5% reserve"
          targetSize=$targetSizeAfterUpgradePlus5Percent
          theCommand="chfs -a size=$targetSize /usr"
        else
          echo
          echo "Insufficient space on hdisk0 to increase the size of /usr with 5% reserve"
          echo
          if (( $totalAvailableOnHdisk0 >= $targetSizeAfterUpgradePlus1Percent))
          then
            echo "INFO ($scriptName): ready to increase the size of /usr to the required size + 1% reserve"
            targetSize=$targetSizeAfterUpgradePlus1Percent
            theCommand="chfs -a size=$targetSize /usr"
          else
            echo "Insufficient space on hdisk0 to increase the size of /usr with 1% reserve"
            theCommand='insufficient space'
          fi
        fi
      fi
    fi
  fi

  echo
  if [ "$theCommand" = 'insufficient space' ]
  then
    echo "NSM-95201-E: ERROR ($scriptName) - there is insufficient space available to increase"
    echo " the size of the /usr filesystem to accomodate this upgrade. Corrective"
    echo " action is required. Contact IBM support for assistance"
    echo 
    echo "exiting ...."
    echo 
    exit 1
  elif [ "$theCommand" = 'space currently OK' ]
  then
    : 
  else
    if (( $totalAvailableOnHdisk1 >= $targetSize))
    then
      echo
      echo "INFO ($scriptName): issuing command: $theCommand"
      eval $theCommand
      chfsRc=$?
      if (( $chfsRc != 0 ))
      then
        echo "NSM-95202-E: ERROR ($scriptName) - failure changing the size of /usr filestystem"
        echo "  corrective action required. Contact IBM Support"
        echo 
        echo "exiting ...."
        echo 
        exit 1
      fi
    else
      echo "NSM-95203-E: ERROR ($scriptName) - the amount of space on hdisk1: $totalAvailableOnHdisk1 512 byte blocks"
      echo " is not sufficient for the required allocation of $targetSize 512 byte blocks."
      echo " The amount of space on hdisk0: $totalAvailableOnHdisk0 512 byte blocks is sufficient"
      echo " Disks hdisk0 and hdisk1 should always have an equivalent amount of free space"
      echo " Corrective action required. Contact IBM Support"
      echo 
      echo "exiting ...."
      echo
      exit 1
    fi
  fi

} # end adjustSizeOfUsrFilesystem

checkFileSystemUtilization () # this routine is not used due to space concerns for 4.5GB operating system disks
{
  fileSystemList='/:50 /usr:80 /tmp:50 /var:50'
  allFilesystemsExist='yes'
  allFilesystemsUtilzationOk='yes'
  echo
  for currentFileSystemCombo in $fileSystemList
  do

    # --------------------------------------------
    # Parse information for the current filesystem
    # --------------------------------------------
    currentFileSystem=`echo $currentFileSystemCombo | cut -f1 '-d':'`
    currentFileSystemThreshHold=`echo $currentFileSystemCombo | cut -f2 '-d':'`

    # -------------------------------------
    # verify the filesystem exists
    # -------------------------------------
    df $currentFileSystem >/dev/null 2>&1
    dfRc=$?
    if (( $dfRc > 0 ))
    then
      allFilesystemsExist='no'
      echo "NSM-95204-E: ERROR: ($scriptName) - The filesystem '$currentFileSystem' does not exist"
    else
      # -------------------------------------
      # verify filesystem utilization
      # -------------------------------------
      fileSystemDfString=`df $currentFileSystem | grep -v 'Filesystem'`
      fileSystemUtilization=`echo $fileSystemDfString | awk '{print $4}' | cut -f1 -d'%'`
      if (( $fileSystemUtilization > $currentFileSystemThreshHold ))
      then
        echo
        echo "INFO: ($scriptName) - The utilization for filesystem '$currentFileSystem' is $fileSystemUtilization%"
        echo "  This is greater than the maximum utilization of $currentFileSystemThreshHold"
        fileSystemSize512KbBlocks=`echo $fileSystemDfString | awk '{print $2}'`
        fileSystemFree512KbBlocks=`echo $fileSystemDfString | awk '{print $3}'`
        fileSystemUsed512KbBlocks=$(($fileSystemSize512KbBlocks - $fileSystemFree512KbBlocks))
        echo "  The current filesystem size in 512KB blocks is: $fileSystemSize512KbBlocks"
        echo "  The number of free 512KB blocks is: $fileSystemFree512KbBlocks"
        echo "  The number of used 512KB blocks is: $fileSystemUsed512KbBlocks"
        numberOf512BlocksToAdd=$(( (($fileSystemUsed512KbBlocks*100)/$currentFileSystemThreshHold) - $fileSystemSize512KbBlocks + 1 ))
        echo "  The number of 512KB blocks to add is: $numberOf512BlocksToAdd"
        newTotal512Blocks=$(($fileSystemSize512KbBlocks + $numberOf512BlocksToAdd))
        echo "  The new filesystem size in blocks will be: $newTotal512Blocks"
        echo 
        theCommand="chfs -a size=+$numberOf512BlocksToAdd $currentFileSystem"
        echo "INFO: ($scriptName) - issuing command: '$theCommand'"
        eval $theCommand
        theCommandRc=$?
        if (( $theCommandRc > 0 ))
        then
          echo 
          echo "NSM-95205-E: ERROR: ($scriptName) - error trying to change the file size for filesystem: $currentFileSystem"
          allFilesystemsUtilzationOk='no'
        fi
        echo
      else
        echo "INFO: ($scriptName) - The utilization for filesystem '$currentFileSystem' is $fileSystemUtilization%"
        echo "  this is less than the maximum threshold of $currentFileSystemThreshHold%"
      fi
    fi
  done

  if [ "$allFilesystemsExist" = 'no' ]
  then
    echo
    echo "NSM-95206-E: ERROR: ($scriptName) - EXITING, one or more required filesystems do not exist"
    echo 
    echo "exiting ...."
    echo 
    exit 1
  fi

  if [ "$allFilesystemsUtilzationOk" = 'no' ]
  then
    echo
    echo "NSM-95207-E: ERROR: ($scriptName) - EXITING, one or more filesystems exceed the maximum filesystem utilization"
    echo 
    echo "exiting ...."
    echo 
    exit 1
  fi
} # end checkFileSystemUtilization

verifyProperCdromMounted ()
{

# ------------------------------------------------------------------------
# verify the proper cdrom is mounted
# ------------------------------------------------------------------------
doneVerifyingCdrom='no'
while [ "$doneVerifyingCdrom" = 'no' ]
do

  checkForCorrectCdrom=''

  echo
  echo "INFO: ($scriptName) - insert CD with title $cdromLabel"
  echo "                    into the CDrom reader."
  echo
  enterOneToContinue 'always'

  if [ "$developmentOverrideToSkipCdromVerification" = 'yes' ]
  then
    echo
    echo "INFO: ($scriptName) - skipping cdrom verification since developmentOverrideToSkipCdromVerification=$developmentOverrideToSkipCdromVerification"
    return
  fi

  if [ "$mountTarget" = '/dev/cd0' ]
  then
    theCommand='mount /cd'
    echo
    echo "INFO: ($scriptName) - issuing command: $theCommand"
    eval $theCommand
  else
    theCommand="mount -n $nfsHostForH27349 $mountTarget /cd"
    echo
    echo "INFO: ($scriptName) - issuing command: $theCommand"
    eval $theCommand
  fi

  checkIfCdromMounted=`df | grep '/cd' | awk '{print $7}'`
  if [ "$checkIfCdromMounted" = '/cd' ]
  then
    echo
    df /cd
    echo 
  else
    checkForCorrectCdrom='notMounted'
    echo "NSM-95208-E: ERROR: ($scriptName) - unable to mount $cdromLabel"
  fi 

  if [ "$checkForCorrectCdrom" != 'problem' ] && [ "$checkForCorrectCdrom" != 'notMounted' ]
  then
    echo
    echo "INFO: ($scriptName) - verifying that the cdrom is $cdromLabel"
    echo 
    theCommand="ls $fileUniqueToThisCdrom"'*'
    echo 
    echo "INFO: ($scriptName) - Issuing command: $theCommand"
    eval $theCommand
    theRc=$?
    if [ $theRc = 0 ]
    then
      checkForCorrectCdrom='ok'
    else
      checkForCorrectCdrom='problem'
    fi
  fi

  if [ "$checkForCorrectCdrom" = 'notMounted' ]
  then
    echo
    echo '**********************************************************************'
    echo '*'
    echo "* NSM-95209-E: ERROR: ($scriptName) - the cdrom does not appear to be mounted"
    echo '*'
    echo '**********************************************************************'
    echo 

  elif [ "$checkForCorrectCdrom" = 'problem' ]
  then
    echo
    echo '**********************************************************************'
    echo '*'
    echo "* NSM-95210-E: ERROR: ($scriptName) - unable to verify that the $cdromLabel"
    echo "*   is inserted in the cdrom reader. Check the following:"
    echo '*'
    echo "* Verification file is: $fileUniqueToThisCdrom'*'"
    echo '*'
    echo "*  It is likely that an incorrect cdrom is mounted"
    echo '*'
    echo '**********************************************************************'
    echo 
    theCommand='unmount /cd'
    echo 
    echo "Issuing command: $theCommand"
    echo
    eval $theCommand
    theCommandRc=$?
    echo
    if (( $theCommandRc == 0 ))
    then
      echo "The incorrect cdrom was unmounted"
    else
      echo "NSM-95211-E: ERROR unmounting cdrom. Manual intervention required"
      echo 
      echo '  1) determine why cdrom cannot be unmounted'
      echo '     one likely cause is the directory is cded into'
      echo '     from some other window'
      echo "  2) manually issue the command: '$theCommand'"
    fi

  else
    doneVerifyingCdrom='yes'
    echo
    echo "INFO: ($scriptName) - verification that $cdromLabel"
    echo " is inserted in Cdrom drive is successfull. Continuing ...."
  fi

done 

} # end verifyProperCdromMounted

unmountCd ()
{
  theCommand='umount /cd'
  echo
  echo "INFO: ($scriptName) - issuing command: $theCommand"
  eval $theCommand
  umountRc=$? 
  if [ $umountRc != 0 ]
  then
    echo
    echo "NSM-95212-E: ERROR: ($scriptName) - unable to unmount $cdromLabel"
    echo 
    echo "It is required that you determine the reason the cdrom cannot be unmounted."
    echo "A typical cause is that someone has issued the cd command to change to this"
    echo "directory. This prevents the unmount from working."
    echo 
    echo "Once you have removed the cause for the unmount failure, the /cd filesystem"
    echo "must be unmounted from another window with the command 'unmount /cd'"
    echo 
    echo "Indicate what action to take"
    echo
    sleep 1
    PS3="Enter choice: "
    select userResponse in \
        'continue - the /cd filesystem is unmounted' \
        'exit - unable to unmount the /cd filesystem'
    do
      case $REPLY in
        1) echo "User specified option 1: 'continue - the /cd filesystem is unmounted'"
           checkIfCdromMounted=`df | grep '/cd' | awk '{print $7}'`
           if [ "$checkIfCdromMounted" = '/cd' ]
           then
             echo
             echo "NSM-95213-E: ERROR: ($scriptName) - the /cd rom filesystem is still mounted"
             echo 
             echo 'Your response that the cdrom filesystem was unmounted is incorrect'
             echo
             theCommand='df'
             echo "Issuing command: $theCommand"
             echo 
             eval $theCommand
             echo 
             echo "Exiting .................."
             echo 
             exit 1
           fi
           ;;
        2) echo
           echo "User specified option 2: 'exit - unable to unmount the /cd filesystem"
           echo "exiting ......"
           exit
           ;;
        *) echo 'Invalid Selection.'
           ;;
      esac
 
      if [[ -n $userResponse ]] ; then
        break
      fi
    done
  fi

} # end unmountCd

verifyToc ()
{
  listOfDirectoryWhichMustContainToc=$*
  echo 
  tocValidationPassed='yes'
  for currentDirectory in $listOfDirectoryWhichMustContainToc
  do 
    if [ -d $currentDirectory ]
    then
      cd $currentDirectory >/dev/null 2>&1
      if [ -f .toc ]
      then
        echo
        theCommand="installp -ld `pwd`"
        echo "INFO ($scriptName) -  Issuing command: '$theCommand' to verify .toc file integrity"
        echo "   Note: the output is redirected to /dev/null"
        echo 
        eval $theCommand >/dev/null 2>&1
        installpRc=$?
        if (( $installpRc == 0))
        then
          :
        else
          tocValidationPassed='no'
          echo 
          echo "NSM-95214-E: ERROR ($scriptName) - error from installp using the '.toc' file to list the"
          echo "  directory contents. The return code from installp is: $installpRc"
        fi
      else
        tocValidationPassed='no'
        echo 
        echo "NSM-95215-E: ERROR ($scriptName) - the '.toc' file not found in the current directory"
      fi
      cd - >/dev/null 2>&1
    else
      tocValidationPassed='no'
      echo 
      echo "NSM-95216-E: ERROR ($scriptName) - the directory '$currentDirectory' does not exist"
      echo "   This is a directory that was specified to contain a '.toc' file"
    fi
  done
  echo 
  if [ "$tocValidationPassed" = 'yes' ]
  then
    echo
    echo "INFO ($scriptName) - all '.toc' validation passed for current cdrom"
  else
    echo
    echo "NSM-95217-E: ERROR ($scriptName) - one or more '.toc' validation errors occured for this cdrom"
    echo "  An I/O error is suspected on the Cdrom. Call IBM support for assistance"
    echo
    unmountCd
    echo
    echo "exiting ...."
    echo 
    exit 1
  fi
} # end verifyToc

checkFilesetLevel ()
{

# --------------------------------------------------
# Capture positional parms
# --------------------------------------------------
fileset="$1"
requiredLevel="$2"
echo ${requiredLevel} | sed -e "s/\./ /g" | read Level1Required Level2Required Level3Required Level4Required Level5Required
echo
echo "Checking for fileset: '$fileset' at level: '$requiredLevel' or higher"


# --------------------------------------------------
# Capture and parse information for the fileset to
# determine if it is installed and at what level
# --------------------------------------------------
LSLPP=`lslpp -L $fileset 2>/dev/null`
lslppRc=$?
if [ $lslppRc = 0 ]
then
  filesetFound='no'
  filesetLevelFound='no'
  for curWord in $LSLPP
  do
    if [ "$filesetFound" = 'yes' ]
    then
      filesetLevel="$curWord"
      filesetLevelFound='yes'
      break
    elif [ "$curWord" = "$fileset" ]
    then
      filesetFound='yes'
    fi
  done
fi
# --------------------------------------------------
# If fileset was found determine if it is installed
# at the required level or higher
# --------------------------------------------------
returnString='undetermined' # set return code, will be changed to another value when state is determined
if [ "$filesetFound" = 'yes' ] && [ "$filesetLevelFound" = 'yes' ]
then
  echo "Fileset: '$fileset' found at level: '$filesetLevel'"

  echo ${filesetLevel} | sed -e "s/\./ /g" | read Level1captured Level2captured Level3captured Level4captured Level5captured

  # -------------------------------------------
  # Determine if level 1 is at a suffcicient level
  # if higher than the required level 1 checking is done
  # if lower than the required level 1 checking is done
  # -------------------------------------------
  if [ $Level1captured -gt $Level1Required ] # check level 1
  then
    returnString='ok' # level is ok, all done checking
  else
    if [ $Level1captured -lt $Level1Required ] # check level 1
    then
      returnString='mismatch at level 1' # level 1 is not ok, all done checking
    fi
  fi

  # -------------------------------------------
  # Determine if level 2 is at a suffcicient level
  # if higher than the required level 2 checking is done
  # if lower than the required level 2 checking is done
  # -------------------------------------------
  if [ "$Level2captured" = '' ]   
  then
    : # fall through, this level is not required
  elif [ "$returnString" = 'undetermined' ]
  then
    if [ $Level2captured -gt $Level2Required ] # check level 2
    then
      returnString='ok' # level is ok, all done checking
    else
      if [ $Level2captured -lt $Level2Required ] # check level 2
      then
        returnString='mismatch at level 2' # level 2 is not ok, all done checking
      fi
    fi
  fi

  # -------------------------------------------
  # Determine if level 3 is at a suffcicient level
  # if higher than the required level 3 checking is done
  # if lower than the required level 3 checking is done
  # -------------------------------------------
  if [ "$Level3captured" = '' ]
  then
    : # fall through, this level is not required
  elif [ "$returnString" = 'undetermined' ]
  then
    if [ $Level3captured -gt $Level3Required ] # check level 3
    then
      returnString='ok' # level is ok, all done checking
    else
      if [ $Level3captured -lt $Level3Required ] # check level 3
      then
        returnString='mismatch at level 3' # level 3 is not ok, all done checking
      fi
    fi
  fi

  # -------------------------------------------
  # Determine if level 4 is at a suffcicient level
  # if higher than the required level 4 checking is done
  # if lower than the required level 4 checking is done
  # -------------------------------------------
  if [ "$Level4captured" = '' ]
  then
    : # fall through, this level is not required
  elif [ "$returnString" = 'undetermined' ]
  then
    if [ $Level4captured -gt $Level4Required ] # check level 4
    then
      returnString='ok' # level is ok, all done checking
    else
      if [ $Level4captured -lt $Level4Required ] # check level 4
      then
        returnString='mismatch at level 4' # level 4 is not ok, all done checking
      fi
    fi
  fi

  # -------------------------------------------
  # Determine if level 5 is at a suffcicient level
  # if higher than the required level 5 checking is done
  # if lower than the required level 5 checking is done
  # -------------------------------------------
  if [ "$Level5captured" = '' ]
  then
    : # fall through, this level is not required
  elif [ "$returnString" = 'undetermined' ]
  then
    if [ $Level5captured -gt $Level5Required ] # check level 5
    then
      returnString='ok' # level is ok, all done checking
    else
      if [ $Level5captured -lt $Level5Required ] # check level 5
      then
        returnString='mismatch at level 5' # level 5 is not ok, all done checking
      fi
    fi
  fi

  if [ "$returnString" = 'undetermined' ]
  then
    returnString='ok' # level is ok, all done checking
  fi

else
  echo "Fileset: '$fileset' NOT FOUND"
  returnString='lpp not found' # lpp not found
fi

if [ "$returnString" = 'ok' ]
then
  returnCode=0
elif [ "$returnString" = 'lpp not found' ]
then
  returnCode=2
else
  returnCode=1
fi

return $returnCode

} # end checkFilesetLevel

updateAll  ()
{
  installDirectory="$1"
  estimatedTime="$2"  # estimated installation time in minutes
  installRatherThanUpdate=$3

  # ------------------------------------------------------------------------
  # do the updates
  # ------------------------------------------------------------------------
  if [ "$mountTarget" != '/dev/cd0' ]
  then
    echo
#   theCommand="inutoc $installDirectory"
#   echo "INFO: ($scriptName) - issuing command: $theCommand"
#   echo 
#   eval $theCommand
#   echo
  fi
  echo
  echo "INFO: ($scriptName) - the installation is logged in: $logFileName"
  echo "INFO: ($scriptName) - the installation log can be monitored with the command:"
  echo "                    'tail -f $logFileName'"
  echo "                    CNTL-C to terminate the tail command" 
  echo
  echo "INFO: ($scriptName) - estimated installation time is: $estimatedTime minutes"
  echo
  if [ "$installRatherThanUpdate" = '' ]
  then
    theCommand="/usr/lib/instl/sm_inst installp_cmd -a -d '$installDirectory' -f '_update_all' '-c' '-N' '-g' '-X' '-S'"
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): before installp command: `date`" | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): Issuing command $theCommand"  | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    eval $theCommand >> $logFileName 2>&1
    installp_cmd_Rc=$?
    echo | tee -a $logFileName 2>&1
    echo "The installp_cmd return code is: $installp_cmd_Rc" | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): after installp command: `date`" | tee -a $logFileName 2>&1
  else
    theCommand=" /usr/lib/instl/sm_inst installp_cmd -a -Q -d '$installDirectory' -f '_all_latest'  '-c' '-N' '-g' '-X' '-G'"
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): before installp command: `date`" | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): Issuing command $theCommand"  | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    eval $theCommand >> $logFileName 2>&1
    installp_cmd_Rc=$?
    echo | tee -a $logFileName 2>&1
    echo "The installp_cmd return code is: $installp_cmd_Rc" | tee -a $logFileName 2>&1
    echo | tee -a $logFileName 2>&1
    echo "INFO: ($scriptName): after installp command: `date`" | tee -a $logFileName 2>&1
  fi

  echo
  echo "INFO: ($scriptName) OSlevel after installing $installDirectory is: `oslevel`"

} # end updateAll

updateSoftware ()
{
  echo
  echo "----------------------------------------------------------------------"
  # --------------------------------------------------------------------------
  # Determine if cdrom 1 should be mounted
  # --------------------------------------------------------------------------
  if [ "$skipAixInstallCdrom1_aix_base_install" = 'yes' ] && [ "$skipAixInstallCdrom1" = 'yes' ]
  then
    echo 
    echo "INFO ($scriptName) - skipping mounting / processing AIX installation cdrom 1 since:"
    echo 
    echo "  skipAixInstallCdrom1_aix_base_install=$skipAixInstallCdrom1_aix_base_install"
    echo "  skipAixInstallCdrom1=$skipAixInstallCdrom1"
    echo
    enterOneToContinue
  else
    # ------------------------------------------------------------------------
    # verify that AIX installation cdrom 1 of 4 is mounted
    # ------------------------------------------------------------------------
    fileUniqueToThisCdrom="/cd/usr/sys/inst.images/sysmgt.msg.zh_CN.websm"
    cdromLabel="'AIX V4.3.3 Volume 1 of 4'"
    if [ "$nfsHostForH27349" = '' ]
    then
      mountTarget='/dev/cd0'
    else
      mountTarget="$aix_installation_cdrom1_altdir" 
    fi
    verifyProperCdromMounted 
    verifyToc '/cd/usr/sys/inst.images'

    # ------------------------------------------------------------------------
    # install the required filesets from AIX installation cdrom 1 of 4
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipAixInstallCdrom1_aix_base_install" = 'yes' ]
    then
      echo 
      echo "INFO ($scriptName) - skipping AIX base installation from cdrom 1 since:"
      echo 
      echo "  skipAixInstallCdrom1_aix_base_install=$skipAixInstallCdrom1_aix_base_install"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.install.first.log"
      theCommand="installp -acgXd $aix_installation_cdrom1_dir bos.rte.install 4.3.3.0  \
          bos.diag 4.3.3.0 \
          devices.scsi.tape.rte 4.3.3.0 \
          devices.fcp.disk 4.3.3.0 \
          devices.fcp.disk.array 4.3.3.0 \
          devices.fcp.tape 4.3.3.0 \
          devices.pci.pci 4.3.3.0 \
          devices.scsi.tape.rte 4.3.3.0 \
          devices.scsi.scarray 4.3.3.0 \
          devices.pci.df1000f7 4.3.3.0 \
          devices.common.IBM.fc 4.3.3.0 "
      echo | tee $logFileName
      echo "INFO: ($scriptName) - Issuing command $theCommand" | tee -a $logFileName
      echo "INFO: ($scriptName) - the installation is logged in: $logFileName"
      echo "INFO: ($scriptName) - estimated installation time is: 7 minutes"
      echo | tee -a $logFileName
      eval $theCommand >> $logFileName 2>&1
      installpRc=$?
      echo | tee -a $logFileName 2>&1
      echo "The installp return code is: $installpRc" | tee -a $logFileName 2>&1
      installRC=$?
      if (( $installpRc > 0 ))
      then
        unmountCd
        echo "NSM-95218-E: ERROR: ($scriptName) - non zero return code from install first phase"
        echo " review $logFileName to determine reason for the non zero return code"
        echo
        echo " exiting ..."
        exit 1
      fi
    fi
  
    # ------------------------------------------------------------------------
    # invoke update all for AIX installation cdrom 1 of 4
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipAixInstallCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for AIX installation cdrom 1 since:"
      echo 
      echo "  skipAixInstallCdrom1=$skipAixInstallCdrom1"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.aixdisk1.log"
      updateAll "$aix_installation_cdrom1_dir" '50-60'
      unmountCd
    fi
  fi
  
  # ------------------------------------------------------------------
  # Process AIX installation cdrom 2 as required
  # ------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  if [ "$skipAixInstallCdrom2" = 'yes' ]
  then
    echo
    echo "INFO ($scriptName) - skipping update all for AIX installation cdrom 2 since:"
    echo 
    echo "  skipAixInstallCdrom2=$skipAixInstallCdrom2"
    echo
    enterOneToContinue
  else
    # ------------------------------------------------------------------------
    # verify that AIX installation cdrom 2 of 4 is mounted
    # ------------------------------------------------------------------------
    fileUniqueToThisCdrom="/cd/usr/sys/inst.images/PEX_PHIGS.msg.zh_CN"
    cdromLabel="'AIX V4.3.3 Volume 2 of 4'"
    if [ "$nfsHostForH27349" = '' ]
    then
      mountTarget='/dev/cd0'
    else
      mountTarget="$aix_installation_cdrom2_altdir" 
    fi
    verifyProperCdromMounted
    verifyToc '/cd/usr/sys/inst.images'
  
    # ------------------------------------------------------------------------
    # invoke update all for AIX installation cdrom 2 of 4
    # ------------------------------------------------------------------------
    logFileName="$updateLogDir/$buildID.aixdisk2.log"
    updateAll "$aix_installation_cdrom2_dir" '10-15'
    unmountCd
  fi

  # ------------------------------------------------------------------
  # Process AIX installation cdrom 3 as required
  # ------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  if [ "$skipAixInstallCdrom3" = 'yes' ]
  then
    echo
    echo "INFO ($scriptName) - skipping update all for AIX installation cdrom 3 since:"
    echo 
    echo "  skipAixInstallCdrom3=$skipAixInstallCdrom3"
    echo
    enterOneToContinue
  else
    # ------------------------------------------------------------------------
    # verify that AIX installation cdrom 3 of 4 is mounted
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    fileUniqueToThisCdrom="/cd/usr/sys/inst.images/X11.help.zh_CN"
    cdromLabel="'AIX V4.3.3 Volume 3 of 4'"
    if [ "$nfsHostForH27349" = '' ]
    then
      mountTarget='/dev/cd0'
    else
      mountTarget="$aix_installation_cdrom3_altdir" 
    fi
    verifyProperCdromMounted
    verifyToc '/cd/usr/sys/inst.images'

    # ------------------------------------------------------------------------
    # invoke update all for AIX installation cdrom 3 of 4
    # ------------------------------------------------------------------------
    logFileName="$updateLogDir/$buildID.aixdisk3.log"
    updateAll "$aix_installation_cdrom3_dir" '5'
    unmountCd
  fi

  # ------------------------------------------------------------------
  # Process AIX installation cdrom 4 as required
  # ------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  if [ "$skipAixInstallCdrom4" = 'yes' ]
  then
    echo
    echo "INFO ($scriptName) - skipping update all for AIX installation cdrom 4 since:"
    echo 
    echo "  skipAixInstallCdrom4=$skipAixInstallCdrom4"
    echo
    enterOneToContinue
  else
    # ------------------------------------------------------------------------
    # verify that AIX installation cdrom 4 of 4 is mounted
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    fileUniqueToThisCdrom="/cd/usr/sys/inst.images/X11.help.ZH_CN"
    cdromLabel="'AIX V4.3.3 Volume 4 of 4'"
    if [ "$nfsHostForH27349" = '' ]
    then
      mountTarget='/dev/cd0'
    else
      mountTarget="$aix_installation_cdrom4_altdir" 
    fi
    verifyProperCdromMounted
    verifyToc '/cd/usr/sys/inst.images'

    # ------------------------------------------------------------------------
    # invoke update all for AIX installation cdrom 4 of 4
    # ------------------------------------------------------------------------
    logFileName="$updateLogDir/$buildID.aixdisk4.log"
    updateAll "$aix_installation_cdrom4_dir" '1'
    unmountCd
  fi

  # --------------------------------------------------------------------------
  # Determine if NSM update cdrom 1 should be mounted
  # --------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  if [ "$skipAixBaseDocOn3466UpdateCdrom1" = 'yes' ] && \
     [ "$skipNsmPackageOn3466UpdateCdrom1" = 'yes' ] && \
     [ "$skipNsmDocPackageOn3466UpdateCdrom1" = 'yes' ] 
  then
    echo 
    echo "INFO ($scriptName) - skipping mounting / processing 3466 update cdrom 1 since:"
    echo 
    echo "  skipAixBaseDocOn3466UpdateCdrom1=$skipAixBaseDocOn3466UpdateCdrom1"
    echo "  skipNsmPackageOn3466UpdateCdrom1=$skipNsmPackageOn3466UpdateCdrom1"
    echo "  skipNsmDocPackageOn3466UpdateCdrom1=$skipNsmDocPackageOn3466UpdateCdrom1"
    echo
    enterOneToContinue
  else
    # ------------------------------------------------------------------------
    # verify that 3466 update cdrom 1  mounted
    # ------------------------------------------------------------------------
    fileUniqueToThisCdrom="/cd/nsm.4.1.0.1/nsm.3466"
    cdromLabel='AIX Documentation and 3466 Package Updates, EC: H27349 CD 1 of 2'
    if [ "$nfsHostForH27349" = '' ]
    then
      mountTarget='/dev/cd0'
    else
      mountTarget="$nsm_update_cdrom1_altdir" 
    fi
    verifyProperCdromMounted
    verifyToc '/cd/baseDoc/usr/sys/inst.images' '/cd/nsm.4.1.0.1' '/cd/nsm.html.3.1.0.33'

    # ------------------------------------------------------------------------
    # update the AIX Base Documentation
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipAixBaseDocOn3466UpdateCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for AIX Base Documentation on 3466 Update Cdrom 1:"
      echo 
      echo "  skipAixBaseDocOn3466UpdateCdrom1=$skipAixBaseDocOn3466UpdateCdrom1"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.aixBaseDocumentation.log"
      updateAll "$nsm_update_cdrom1_aix_baseDoc" '75-90'
    fi

    # ------------------------------------------------------------------------
    # update the nsm script package
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipNsmPackageOn3466UpdateCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for NSM package on 3466 Update Cdrom 1:"
      echo 
      echo "  skipNsmPackageOn3466UpdateCdrom1=$skipNsmPackageOn3466UpdateCdrom1"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.nsm_package.log"
      updateAll "$nsm_update_cdrom1_nsm_package" '1'
  
      # ------------------------------------------------------
      # verify NSM Package at proper level
      # ------------------------------------------------------
      echo
      echo "----------------------------------------------------------------------"
      reqLevel='4.1.0.0'

      checkFilesetLevel nsm.3466 $reqLevel
      checkFilesetLevelRc1=$?; 

      if (( $checkFilesetLevelRc1 == 0 ))
      then
        successUpdatingNsmPackage='yes'
        echo
        echo "SUCCESS ($scriptName) upgrade for NSM package was successfull"  
      else
        echo
        echo "NSM-95221-E: ERROR verifying NSM package installation. The required level for"
        echo "  these filesets is $reqLevel"
        echo
        echo "Corrective action may be required"
        echo
        echo "Diagnostic date in file: /tmp/updateEc_""$scriptName/<mmdd_hhmm>.nsm_package.log"
        echo
        theCommand='lslpp -l nsm.3466'
        echo "Issuing command: $theCommand"
        echo 
        eval $theCommand
        echo
        echo "This script will exit."
        enterOneToContinue 'always'
        echo "Exiting ......."
        unmountCd
        exit 1
        successUpdatingNsmPackage='no'
      fi
    fi

    # ------------------------------------------------------------------------
    # update the nsm doc package
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skip3466UpdateCdrom1_nsmdoc_package" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for NSM doc package on 3466 Update Cdrom 1:"
      echo 
      echo "  skip3466UpdateCdrom1_nsmdoc_package=$skip3466UpdateCdrom1_nsmdoc_package"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.nsmdoc_package.log"
      updateAll "$nsm_update_cdrom1_nsmdoc_package" '1'

      # ------------------------------------------------------
      # verify NSM Doc Package at proper level
      # ------------------------------------------------------
      echo
      echo "----------------------------------------------------------------------"
      reqLevel='3.1.0.33'

      checkFilesetLevel nsm.html.en_US.data $reqLevel
      checkFilesetLevelRc1=$?;

      if (( $checkFilesetLevelRc1 == 0 ))
      then
        successUpdatingNsmDocPackage='yes'
        echo
        echo "SUCCESS ($scriptName) upgrade for NSM Doc package was successfull"
      else
        echo
        echo "NSM-95222-E: ERROR verifying NSM Doc package installation. The required level for"
        echo "  these filesets is $reqLevel"
        echo
        echo "Corrective action may be required"
        echo
        echo "Diagnostic date in file: /tmp/updateEc_""$scriptName/<mmdd_hhmm>.nsmdoc_package.log"
        echo
        theCommand='lslpp -l nsm.html.en_US.data'
        echo "Issuing command: $theCommand"
        echo
        eval $theCommand
        echo
        echo "This script will exit."
        enterOneToContinue 'always'
        echo "Exiting ......."
        unmountCd
        exit 1
        successUpdatingNsmDocPackage='no'
      fi
    fi
    unmountCd
  fi

  # --------------------------------------------------------------------------
  # Determine if 3466 update cdrom 2 should be mounted
  # --------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  if [ "$skipAixMaintenance4334On3466UpdateCdrom2" = 'yes' ] && \
     [ "$skipServiceDirectorOn3466UpdateCdrom2" = 'yes' ]   && \
     [ "$skipAtapeOn3466UpdateCdrom2" = 'yes' ] && \
     [ "$skipAtlddOn3466UpdateCdrom2" = 'yes' ] 
  then
    echo 
    echo "INFO ($scriptName) - skipping mounting / processing 3466 update cdrom 2 since:"
    echo 
    echo "  skipAixMaintenance4334On3466UpdateCdrom2=$skipAixMaintenance4334On3466UpdateCdrom2"
    echo "  skipServiceDirectorOn3466UpdateCdrom2=$skipServiceDirectorOn3466UpdateCdrom2"
    echo "  skipAtlddOn3466UpdateCdrom2=$skipAtlddOn3466UpdateCdrom2"
    echo "  skipAtapeOn3466UpdateCdrom2=$skipAtapeOn3466UpdateCdrom2"
    echo
    enterOneToContinue
  else
    # ------------------------------------------------------------------------
    # verify that 3466 update cdrom 2 is mounted
    # ------------------------------------------------------------------------
    fileUniqueToThisCdrom="/cd/serviceDirector.3.0.0.6/servdir.installp"
    cdromLabel='AIX Maintenance 4.3.3.4 (APAR U472081), EC: H27349 CD 2 of 2'
    if [ "$nfsHostForH27349" = '' ]
    then
      mountTarget='/dev/cd0'
    else
      mountTarget="$nsm_update_cdrom2_altdir"
    fi
    verifyProperCdromMounted
    verifyToc '/cd/usr/sys/inst.images' '/cd/atape.5.3.9.0' '/cd/atldd.4.1.5.0'  '/cd/serviceDirector.3.0.0.6'

    # ------------------------------------------------------------------------
    # update the recommended AIX fixes for AIX 4.3.3.0 to AIX 4.3.3.4
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipAixMaintenance4334On3466UpdateCdrom2" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for Recommended AIX Fixes (4.3.3.0 to 4.3.3.4) on 3466 Update Cdrom 2 since:"
      echo 
      echo "  skipAixMaintenance4334On3466UpdateCdrom2=$skipAixMaintenance4334On3466UpdateCdrom2"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.recommended_aix_fixes_4334.log"
      updateAll "$nsm_update_cdrom2_recommendedMaintenance4334" '20-25'
    fi

    # ------------------------------------------------------------------------
    # update service director
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipServiceDirectorOn3466UpdateCdrom2" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for service director on 3466 Update Cdrom 2 since:"
      echo 
      echo "  skipServiceDirectorOn3466UpdateCdrom2=$skipServiceDirectorOn3466UpdateCdrom2"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.serviceDirector.log"
      updateAll "$nsm_update_cdrom2_serviceDirector" '1'
  
      # ------------------------------------------------------
      # verify Service Director Package at proper level
      # ------------------------------------------------------
      echo
      echo "----------------------------------------------------------------------"
      reqLevel='3.0.0.6'

      checkFilesetLevel serv_dir.obj $reqLevel
      checkFilesetLevelRc1=$?; 

      if (( $checkFilesetLevelRc1 == 0 ))
      then
        successUpdatingServiceDirectorPackage='yes'
        echo
        echo "SUCCESS ($scriptName) upgrade for service director was successfull"  
      else
        echo
        echo "NSM-95223-E: ERROR verifying service director. The required level for"
        echo "  these filesets is $reqLevel"
        echo
        echo "Corrective action may be required"
        echo
        echo "Diagnostic date in file: /tmp/updateEc_""$scriptName/<mmdd_hhmm>.serviceDirector.log"
        echo
        theCommand='lslpp -l serv_dir.obj'
        echo "Issuing command: $theCommand"
        echo 
        eval $theCommand
        echo
        echo "This script will exit."
        enterOneToContinue 'always'
        echo "Exiting ......."
        unmountCd
        exit 1
        successUpdatingServiceDirectorPackage='no'
      fi
    fi

    # ------------------------------------------------------------------------
    # update atldd
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipAtlddOn3466UpdateCdrom2" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for atldd on 3466 Update Cdrom 2 since:"
      echo 
      echo "  skipAtlddOn3466UpdateCdrom2=$skipAtlddOn3466UpdateCdrom2"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.atldd.log"
      updateAll "$nsm_update_cdrom2_atldd" '1'
  
      # ------------------------------------------------------
      # verify atldd Package at proper level
      # ------------------------------------------------------
      echo
      echo "----------------------------------------------------------------------"
      reqLevel='4.1.5.0'

      checkFilesetLevel atldd.driver $reqLevel
      checkFilesetLevelRc1=$?; 

      if (( $checkFilesetLevelRc1 == 0 ))
      then
        successUpdatingAtlddPackage='yes'
        echo
        echo "SUCCESS ($scriptName) upgrade for atldd package was successfull"  
      else
        echo
        echo "NSM-95225-E: ERROR verifying atldd installation. The required level for"
        echo "  these filesets is $reqLevel"
        echo
        echo "Corrective action may be required"
        echo
        echo "Diagnostic date in file: /tmp/updateEc_""$scriptName/<mmdd_hhmm>.atldd.log"
        echo
        theCommand='lslpp -l atldd.driver'
        echo "Issuing command: $theCommand"
        echo 
        eval $theCommand
        echo
        echo "This script will exit."
        enterOneToContinue 'always'
        echo "Exiting ......."
        unmountCd
        exit 1
        successUpdatingAtlddPackage='no'
      fi
    fi

    # ------------------------------------------------------------------------
    # update Atape
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipAtapeOn3466UpdateCdrom2" = 'yes' ]
    then
      echo 
      echo "INFO ($scriptName) - skipping installing Atape on 3466 Update Cdrom 2 since:"
      echo 
      echo "  skipAtapeOn3466UpdateCdrom2=$skipAtapeOn3466UpdateCdrom2"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.atape.log"
      updateAll "$nsm_update_cdrom2_Atape" '2'
  
      # ------------------------------------------------------
      # verify Atape Package at proper level
      # ------------------------------------------------------
      echo
      echo "----------------------------------------------------------------------"
      reqLevel='5.3.9.0'

      checkFilesetLevel Atape.driver $reqLevel 
      checkFilesetLevelRc1=$?; 

      if (( $checkFilesetLevelRc1 == 0 ))
      then
        successUpdatingAtapePackage='yes'
        echo
        echo "SUCCESS ($scriptName) upgrade for Atape package was successfull"  
      else
        echo
        echo "NSM-95224-E: ERROR verifying Atape package installation. The required level for"
        echo "  these filesets is $reqLevel"
        echo
        echo "Corrective action may be required"
        echo
        echo "Diagnostic date in file: /tmp/updateEc_""$scriptName/<mmdd_hhmm>.atape.log"
        echo
        theCommand='lslpp -l Atape.driver'
        echo "Issuing command: $theCommand"
        echo 
        eval $theCommand
        echo
        echo "This script will exit."
        enterOneToContinue 'always'
        echo "Exiting ......."
        unmountCd
        exit 1
        successUpdatingAtapePackage='no'
      fi
    fi
    unmountCd
  fi


  # --------------------------------------------------------------------------
  # Determine if AIX maintenance cdrom for AIX 4.3.3.x should be mounted
  # --------------------------------------------------------------------------
  if [ "$skipAixMaintenance433xOnAixMaintenanceCdrom1" = 'yes' ] && \
     [ "$skipMiscAixMaintenanceOnOnAixMaintenanceCdrom1" = 'yes' ] && \
     [ "$skipSsaMaintenanceOnAixMaintenanceCdrom1" = 'yes' ]
  then
    echo
    echo "INFO ($scriptName) - skipping mounting / processing 3466 update AIX 4.3.3.x cdrom 1 since:"
    echo
    echo "  skipAixMaintenance433xOnAixMaintenanceCdrom1=$skipAixMaintenance433xOnAixMaintenanceCdrom1"
    echo "  skipMiscAixMaintenanceOnOnAixMaintenanceCdrom1=$skipMiscAixMaintenanceOnOnAixMaintenanceCdrom1"
    echo "  skipSsaMaintenanceOnAixMaintenanceCdrom1=$skipSsaMaintenanceOnAixMaintenanceCdrom1"
    enterOneToContinue
  else
    # ------------------------------------------------------------------------
    # verify that AIX Maintenance update cdrom 1 is mounted
    # ------------------------------------------------------------------------
    fileUniqueToThisCdrom="/cd/usr/sys/inst.images/printers.hpJetDirect.attach.usr"
    cdromLabel="AIX Maintenance 4.3.3.x, EC: H27349 CD 1 of 1"
    if [ "$nfsHostForH27349" = '' ]
    then
      mountTarget='/dev/cd0'
    else
      mountTarget="$nsm_aix433x_update_cdrom1_altdir"
    fi

    verifyProperCdromMounted

    if [ -d $ssa_aixMaintenance433x_update_cdrom1 ]
    then
      ssaTocCheckDir="$ssa_aixMaintenance433x_update_cdrom1"
    else
      ssaTocCheckDir=''
      echo
      echo "INFO ($scriptName) - skipping installation for SSA software. The required directory"
      echo "  /cd/ssa does not exist. This can result from the use of a downlevel cdrom. The"
      echo "  The downlevel cdrom has part number '19P1256'. This is the cdrom that is currently"
      echo "  mounted. Please check the part number when this cdrom is unmounted."
      echo  
      echo "  Contact customer support for assistance in completing the ssa upgrade if the"
      echo "  backlevel cdrom is identified."
      echo
      echo "  If the cdrom is not downlevel this should be reported as a problem"
      echo
      enterOneToContinue
    fi
    verifyToc '/cd/usr/sys/inst.images' '/cd/pieceMealFixes' "$ssaTocCheckDir"

    # ------------------------------------------------------------------------
    # update the recommended AIX fixes for AIX 4.3.3.4 to AIX 4.3.3.x
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipAixMaintenance433xOnAixMaintenanceCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update all for Recommended AIX Fixes 4.3.3.x on 3466 Update Cdrom 1 of 1 since:"
      echo 
      echo "  skipAixMaintenance433xOnAixMaintenanceCdrom1=$skipAixMaintenance433xOnAixMaintenanceCdrom1"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.recommended_aix_fixes_4335.log"
      updateAll "$nsm_aixMaintenance433x_update_cdrom1" '20-25'
    fi

    # ------------------------------------------------------------------------
    # update the misc AIX fixes
    # ------------------------------------------------------------------------
    echo
    echo "----------------------------------------------------------------------"
    if [ "$skipMiscAixMaintenanceOnOnAixMaintenanceCdrom1" = 'yes' ]
    then
      echo
      echo "INFO ($scriptName) - skipping update misc AIX fixes on 3466 Update Cdrom 1 of 1 since:"
      echo 
      echo "  skipMiscAixMaintenanceOnOnAixMaintenanceCdrom1=$skipMiscAixMaintenanceOnOnAixMaintenanceCdrom1"
      echo
      enterOneToContinue
    else
      logFileName="$updateLogDir/$buildID.misc_aix_fixes.log"
      updateAll "$miscMaintenance_aixMaintenance433x_update_cdrom1" '5'
    fi

    # ------------------------------------------------------------------------
    # update the ssa files
    # ------------------------------------------------------------------------
    if [ -d $ssa_aixMaintenance433x_update_cdrom1 ]
    then
      echo
      echo "----------------------------------------------------------------------"
      if [ "$skipSsaMaintenanceOnAixMaintenanceCdrom1" = 'yes' ]
      then
        echo
        echo "INFO ($scriptName) - skipping installarion of SSA software on 3466 Update Cdrom 1 of 1 since:"
        echo 
        echo "  skipSsaMaintenanceOnAixMaintenanceCdrom1=$skipSsaMaintenanceOnAixMaintenanceCdrom1"
        echo
        enterOneToContinue
      else
        logFileName="$updateLogDir/$buildID.ssa_fixes.log"
        updateAll "$ssa_aixMaintenance433x_update_cdrom1" '5' 'install'

        # ------------------------------------------------------------------------
        # Bring server down if required
        # ------------------------------------------------------------------------
        if [ "$skipBringingServerDown" = 'yes' ]
        then
          echo
          echo "----------------------------------------------------------------------"
          echo
          echo "Skipping bringing the TSM server down"
          echo
          enterOneToContinue
        else
          allServersDown='no'
          while [ "$allServersDown" = 'no' ]
          do
            checkIfDsmservRunning=`ps -elf | grep dsmserv | grep -v grep`
            if [ "$checkIfDsmservRunning" != '' ]
            then
              echo
              echo "INFO ($scriptName) - The TSM server is running (before microcode load):"
              echo 
              ps -elf | grep dsmserv | grep -v grep 
              echo
              echo "Hit '1' to continue and bring the server down"
              echo 
              enterOneToContinue 'always'
              endadsm
            else
              allServersDown='yes'
              echo 
              echo "INFO ($scriptName): verified all servers are down"
            fi
          done
        fi

        # ----------------------------------------------------------------------
        # Issue ssa_sesdld command to update enclosure microcode
        # ----------------------------------------------------------------------
        enclosureMicrocodeLevel='0020'
        enclosureMicrocodeFile='/etc/microcode/coral020.hex'
        if [ "$skipSsa_sesdld" = 'yes' ]
        then
          echo
          echo "INFO: ($scriptName) - skipping 'skipSsa_sesdld ' since skipSsa_sesdld=$skipSsa_sesdld"
        else
          echo
          echo "----------------------------------------------------------------------"
          if [ -f $enclosureMicrocodeFile ]
          then
            theEnclosureList=`lsdev -C | grep enclosure | grep Available | awk '{print $1}'`
            if [ "$theEnclosureList" = '' ]
            then
              : # ok, the drawers may not be coral
            else
              for currentEnclosure in $theEnclosureList
              do
                currentEnclosureMicrocodeLevel=`lscfg -vl $currentEnclosure | grep 'ROS Level and ID' | tr ' ' '_' | tr '.' ' ' | awk '{print $2}'`
                if [ "$currentEnclosureMicrocodeLevel" -ge "$enclosureMicrocodeLevel" ]
                then
                  echo
                  echo "INFO: ($scriptName) - microcode level for $currentEnclosure is: $currentEnclosureMicrocodeLevel"
                  echo "  This is at or later than the required level: $enclosureMicrocodeLevel."
                else
                  echo
                  theCommand="/etc/microcode/ssa_sesdld  -d $currentEnclosure -f $enclosureMicrocodeFile"
                  echo "INFO: ($scriptName) - issuing command: $theCommand"
                  eval $theCommand
                fi
              done
            fi
          else
            echo
            echo "NSM-95226-W: WARNING ($scriptName) - skipping 'skipSsa_sesdld ' since the microcode file"
            echo "   $enclosureMicrocodeFile does not exist"
            echo
            echo "This is an abnormal condition that should be investigated. The enclosure"
            echo " microcode file should have been loaded with the SSA update package."
            echo " The microcode file that should be available is: $enclosureMicrocodeFile"
            echo 
            echo "The correct microcode level for the enclosures is: $enclosureMicrocodeLevel"
            echo "The enclosure microcode level can be checked with a command like: "
            echo "'lscfg -vl enclosure0'"
            echo
            echo "This script will continue. This condition will not prevent succesful"
            echo "completion of this script. Corrective action may be required after the"
            echo "script completes"
            echo
            echo "Contact customer support for assistance if required."
            echo
            enterOneToContinue
          fi
        fi

        # ----------------------------------------------------------------------
        # Issue ssadload command to update ssa disk microcode
        # ----------------------------------------------------------------------
        if [ "$skipSsadload" = 'yes' ]
        then
          echo
          echo "INFO: ($scriptName) - skipping 'ssadload -u' since skipSsadload=$skipSsadload"
        else
          echo
          echo "----------------------------------------------------------------------"  
          echo  
          theCommand='ssadload -u'
          echo "INFO: ($scriptName) - issuing command: $theCommand"
          eval $theCommand
        fi
      fi
    fi
  fi

} # end  updateSoftware

recordNewEcLevel ()
{
 
  InstallDate=`date +"%m%d%Y"`
  InstallTime=`date +"%H%M"`

  ECFILENAME=WebShlLvl.$InstallDate$InstallTime

  printf "$EClevelWhenDone: Filesets installed for EC level \n" > /var/adm/$ECFILENAME
  printf "Date installed is $InstallDate\n" >> /var/adm/$ECFILENAME
  lslpp -L all >> /var/adm/$ECFILENAME

  echo  
  echo "INFO: ($scriptName) - Revised EC level $EClevelWhenDone updated"
  echo
  echo "Issuing 'lsSSlevel' command to confirm"
  echo 
  lsSSlevel 
} # end recordNewEcLevel

getTsmAdminInfo ()
{
  adminInfoVerified='no'
  while [ "$adminInfoVerified" = 'no' ]
  do
    echo
    echo "*************************************************************************"
    echo "* Please enter TSM admin id that has system privileges:" 
    echo "*************************************************************************"
    read adminid
    echo
    echo "*************************************************************************"
    echo "* Please enter a valid password for $adminid:"
    echo "*************************************************************************"
    read adminpw
    echo
    echo "*************************************************************************"
    echo "Is $adminid with password $adminpw correct?"
    echo
    sleep 1
    PS3="Enter choice: "
    select userResponse in \
        'admin information is correct' \
        'admin information is NOT correct. Let me try again' \
        'abort'
    do
      case $REPLY in
        1) echo 
           echo "User responded with option 1: admin information is correct"
           checkComm='yes'
           ;;
        2) echo 
           echo "User responded with option 2: admin information is NOT correct, try again"
           checkComm='no'
           ;;
        3) echo
           echo "exiting ......"
           exit
           ;;
        *) echo
           echo "Invalid response"
           echo
           ;;
      esac
 
      if [[ -n $userResponse ]] ; then
        break
      fi

    done

    if [ "$checkComm" = 'yes' ]
    then
      echo
      theCommand="dsmadmc -id=$adminid -pa=$adminpw -server=ss-m quit"
      echo "Verifying communication with server"
      echo "Issuing command: $theCommand"
      echo
      eval $theCommand > /dev/null 2>&1
      dsmadmcRc=$?
      echo
      if (( $dsmadmcRc == 0 ))
      then
        echo "Communicated succesfully with the server"
        adminInfoVerified='yes'
        invoc="dsmadmc -id=$adminid -pa=$adminpw -server=ss-m"
      else
        echo "Communicated with the server FAILED. Possible reasons are:"
        echo "  -- The userid / password may be incorrect"
        echo "  -- the server may not have had time to start fully"
        echo "  -- the shared memory stanza (ss-m) is missing or invalid"
        echo
        sleepSeconds='60'
        echo 
        echo "Sleeping for $sleepSeconds seconds to allow server time to start"
        echo
        sleep $sleepSeconds
        echo
        echo "Try again"
        echo
      fi

    fi

  done

} # end getTsmAdminInfo


upgradeSystem ()
{

  # ------------------------------------------------------------------------
  # Bring server down if required
  # ------------------------------------------------------------------------
  if [ "$skipBringingServerDown" = 'yes' ]
  then
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo "Skipping bringing the TSM server down"
    echo
    enterOneToContinue
  else
    allServersDown='no'
    while [ "$allServersDown" = 'no' ]
    do
      checkIfDsmservRunning=`ps -elf | grep dsmserv | grep -v grep`
      if [ "$checkIfDsmservRunning" != '' ]
      then
        echo
        echo "INFO ($scriptName) - The TSM server is running (start of upgradeSystem):"
        echo 
        ps -elf | grep dsmserv | grep -v grep 
        echo
        echo "Hit '1' to continue and bring the server down"
        echo 
        enterOneToContinue 'always'
        endadsm
      else
        allServersDown='yes'
        echo 
        echo "INFO ($scriptName): verified all servers are down"
      fi
    done
  fi

  # ------------------------------------------------------------------------
  # Verify filesystem size
  # ------------------------------------------------------------------------
  if [ "$skipUsrFilesystemAdjustment" = 'yes' ]
  then
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo "Skipping adjustment of the /usr filesystem size"
    echo
    enterOneToContinue
    echo
  else
    adjustSizeOfUsrFilesystem
  fi 

  # ------------------------------------------------------------------------
  # create the cdrom filesystem
  # ------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  mkdir /cd >/dev/null 2>&1
  echo 
  str=`cat /etc/filesystems | grep /dev/cd0`
  if [ "$str" = '' ] 
  then
    echo "INFO: ($scriptName) - Creating CDROM filespace /cd."
    theCommand="crfs -v cdrfs -d /dev/cd0 -m /cd -p ro"
    echo "INFO: ($scriptName) - issuing command: $theCommand"
    eval $theCommand
    crfsRc=$?
    if [ "$crfsRc" != 0 ]
    then
      echo "NSM-95219-E: ERROR: ($scriptName) Unable to create cdrom filesystem."
      echo "Resolve problem and repeat this script."
      echo
      echo " exiting ..."
      exit 1
    fi
  else
    echo "INFO: ($scriptName) - CDROM filespace /cd already exists"
  fi

  # ------------------------------------------------------------------------
  # cleanup just in case
  # ------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  theCommand="installp -C" 
  echo "INFO: issuing command: $theCommand in case a previous installp"
  echo "      failed and requires cleanup"
  echo 
  eval $theCommand

  # ------------------------------------------------------
  # update software
  # ------------------------------------------------------
  updateSoftware

  # ------------------------------------------------------
  # verify OS level, if not findout why
  # ------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  echo "INFO ($scriptName) - verifying oslevel upgraded as expected"
  echo
  currentOSlevel=`oslevel`
  requiredOsLevel='4.3.3.0'
  if [ "$currentOSlevel" != "$requiredOsLevel" ] 
  then
    successUpdateingAIX='no'
    echo "NSM-95220-E: ERROR ($scriptName) upgrade to AIX $requiredOsLevel"
    echo "      is incomplete. Running command 'oslevel -l $requiredOsLevel"
    echo "      to find out why"
    echo
    oslevel -l $requiredOsLevel
    echo   
    echo "The above filesets must be updated."
    echo 
    echo "This script will exit."
    enterOneToContinue 'always'
    echo "Exiting ......."
    exit 1
  else
    successUpdateingAIX='yes'
    echo "SUCCESS ($scriptName) upgrade to AIX $requiredOsLevel successfull"  
  fi
  
  # ------------------------------------------------------
  # echo final status
  # ------------------------------------------------------
  echo "SUCCESS: ($scriptName) all updates complete"
  echo
  echo
  echo "----------------------------------------------------------------------"
  echo
  if [ "$isNewEcApplied" = '' ]
  then
    recordNewEcLevel
  else
    echo
    echo "The EC $EClevelWhenDone already recorded as complete"
  fi 
 
  theCommand='lsSSlevel'
  echo
  echo "Issuing command: $theCommand"
  echo 
  eval $theCommand

  # ------------------------------------------------------------------------
  # unmount the /cd filesystem if required 
  # ------------------------------------------------------------------------
  checkIfCdMounted=`df | grep '/cd'`
  if [ "$checkIfCdMounted" != '' ]
  then
    theCommand='unmount /cd'
    echo
    echo "Issuing command: $theCommand"
    echo
    eval $theCommand
    theCommandRc=$?
    echo
    if (( $theCommandRc == 0 ))
    then
      echo "The cdrom was unmounted."
      echo "Remove the cdrom from the reader"
      enterOneToContinue 'always'
    else
      echo "NSM-95227-E: ERROR unmounting cdrom. Manual intervention required"
      echo
      echo '  1) determine why cdrom cannot be unmounted'
      echo '     one likely cause is the directory is cded into'
      echo '     from some other window'
      echo "  2) manually issue the command: '$theCommand'"
      echo "  3) Once the /cd file system is unmounted, remove the cdrom from the reader"
      echo
      echo " exiting ..."
      exit 1
    fi
  fi
  
  # ------------------------------------------------------------------------
  # remove the /cd filesystem
  # ------------------------------------------------------------------------
  if [ "$skipCdFilesystemRemoval" = 'yes' ]
  then
    echo
    echo "INFO: ($scriptName) - skipping removal of /cd filesystem since skipCdFilesystemRemoval=$skipCdFilesystemRemoval"
  else
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo "INFO: ($scriptName) - removing /cd filesystem from /etc/filesystems"
    theCommand='rmfs /cd'
    echo "INFO: ($scriptName) - issuing command: $theCommand"
    eval $theCommand
    rmfsRc="$?"
    if [ $rmfsRc != 0 ]
    then
      echo "NSM-95228-E: ERROR: ($scriptName) - Unable to remove the /cd filesystem"
      echo
      echo " exiting ..."
      exit 1
    fi
  fi

  # ----------------------------------------------------------------------
  # prompt user to reboot the system
  # ----------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  echo "INFO: ($scriptName) - remove the cdrom from the cd reader"
  echo
  echo "INFO: ($scriptName) - issue 'shutdown -Fr' to reboot the system"
  enterOneToContinue

} # end upgradeSystem

 
# ************************************************************************

# -------------------------------------------------------------------------
# Process special options.
#   1) Special options must start with a '-'
#   2) The special option must be adjacent to the '-' and can be any
#      number of characters
#   3) Any number of values can be specified for a special option.
#      If values are processed they must be captured and then shifted left
#   4) All special options must be specified prior to positional parameters
# -------------------------------------------------------------------------

firstCharParm1=`echo $1 | cut -b1`
while [ "$firstCharParm1" = '-' ]
do
  specialOption=`echo $1 | cut -b2-`
  case $specialOption in
    ('h')
      helpOption='yes'
      : ;;
    ('b')
      bypassSSlevelCheck='yes'
      : ;;
    (*)
      echo
      echo "NSM-95230-E: ERROR: ($scriptName) - special option: '$1' not recognized"
      echo "  program terminated"
      exit       
      : ;;
  esac
  shift 1
  firstCharParm1=`echo $1 | cut -b1`
done

# -----------------------------------------------------------------------------
# Write help text as appropriate
# -----------------------------------------------------------------------------
tempHelpFile='/tmp/tempHelpFile'
if [ "$helpOption" = 'yes' ]
then
  echo > $tempHelpFile
  echo "--- Hit SPACE key to move forward one screen" >> $tempHelpFile
  echo "--- Hit 'b' to move backward one screen" >> $tempHelpFile
  echo "--- Hit ENTER to move forward one line" >> $tempHelpFile
  echo  >> $tempHelpFile
  cat $0 | grep \#@ | grep -v 'grep #@'} | cut -b3- >> $tempHelpFile

  clear
  cat $tempHelpFile | grep -v tempHelpFile | more  

  exit
fi

mainRoutine ()
{

  # ------------------------------------------------------------------------
  # say hello
  # ------------------------------------------------------------------------
  echo
  echo "INFO: ENTERING ($scriptName) on `date`"
  echo 

  # ------------------------------------------------------------------------
  # display control variables as appropriatte
  # ------------------------------------------------------------------------
  displayControlVariables

  # ------------------------------------------------------------------------
  # initialize control variables
  # ------------------------------------------------------------------------
  initializeInstallationDirectories

  echo
  echo "----------------------------------------------------------------------"
  echo
  # ----------------------------------------------------------
  # Make sure the system is an RS/6000 H50           
  # ----------------------------------------------------------
  str=$(uname -m | cut -c 9,10 )
  if [ "$str" = "4C" ]
  then
     str=`lsattr -El sys0 | grep modelname | grep H50`
     if [ "$str" = '' ]
     then
       systemIsAnH50='no'
     else
       systemIsAnH50='yes'
     fi
  else
    systemIsAnH50='no'
  fi

  echo
  if [ "$systemIsAnH50" = 'yes' ]
  then
    echo "INFO: ($scriptName) - confirmed that system is an RS/6000 H50"
  else
    echo "NSM-95231-E: ERROR: ($scriptName) - system is not an RS/6000 H50"
    echo 
    theCommand='lsattr -El sys0 | grep modelname | grep H50'
    echo "Issuing command: $theCommand"
    echo 
    eval $theCommand
    echo 
    echo "exiting ....."
  fi

  # ------------------------------------------------------------------------
  # Make sure the system has applied the required pre-requisite EC D19383
  # ------------------------------------------------------------------------
  echo
  echo "----------------------------------------------------------------------"
  echo
  requiredPrereqEC='D19383'
  if [ "$bypassSSlevelCheck" = 'yes' ]
  then
    echo
    echo "INFO: ($scriptName) - skipping check for pre-requisite EC $requiredPrereqEC"
    echo "       per user input"
  else
    isRequiredEcApplied=`lsSSlevel | grep $requiredPrereqEC`
    echo
    if [ "$isRequiredEcApplied" = '' ]
    then
      echo
      echo "NSM-95232-E: ERROR: ($scriptName) - required EC $requiredPrereqEC not applied"
      echo "  exiting ...."
      exit
    else
      echo
      echo "INFO: ($scriptName) - verified that required EC $requiredPrereqEC is applied"
    fi
  fi

  # ------------------------------------------------------------------------
  # See if the new EC already is applied
  # ------------------------------------------------------------------------
  EClevelWhenDone='H27349A'
  requiredPrereqEC='D19383'
  isNewEcApplied=`lsSSlevel | grep $EClevelWhenDone`
  echo
  if [ "$isNewEcApplied" = '' ]
  then
    :
  else
    echo
    echo "----------------------------------------------------------------------"
    echo
    echo "INFO: ($scriptName) - the 'lsSSlevel' command indicates that"
    echo "   EC $EClevelWhenDone is already applied"
    echo
    echo "Indicate what action to take"
    echo
    sleep 1
    PS3="Enter choice: "
    select userResponse in \
        'continue - apply the EC again' \
        'exit'
    do
      case $REPLY in
        1) :
           ;;
        2) echo
           echo "exiting ......"
          exit
           ;;
        *) echo 'Invalid Selection.'
           ;;
      esac

      if [[ -n $userResponse ]] ; then
        break
      fi
    done
  fi


  # ------------------------------------------------------------------------
  # prompt user
  # ------------------------------------------------------------------------
  upgradeSystem

  # ------------------------------------------------------------------------
  # say goodbye
  # ------------------------------------------------------------------------
  echo
  echo "INFO: EXITING ($scriptName) on `date`"
  echo

} # end mainRoutine

# ------------------------------------------------
# Set up logging environement
# ------------------------------------------------
export scriptName='h27349a4.ec'
buildID=`date "+%m%d_%H%M"`
updateLogDir="/tmp/updateEc_""$scriptName"
mkdir -p $updateLogDir >/dev/null 2>&1
consoleOutputFile="$updateLogDir/$buildID.consoleLog"

# ------------------------------------------------
# Check for special cdrom option
# ------------------------------------------------
if [ "$nfsHostForH27349" = '' ]
then
  :
else
  echo 
  echo "**********************************************************************"
  echo "*"
  echo "* A special development override is specified to simulate cdroms"
  echo "*   on the following file server: $nfsHostForH27349"
  echo "*"
  echo "**********************************************************************"
  echo
  enterOneToContinue
  crfs -v cdrfs -d /dev/cd0 -m /cd -p ro >/dev/null 2>&1 # add it can be removed
fi

# ------------------------------------------------
# Invoke the main routine to do the work
# ------------------------------------------------
clear
echo
echo "----------------------------------------------------------------------"
echo
echo "INFO: ($scriptName): console output is logged in the file: $consoleOutputFile" 
echo

# ----------------------------------------------------------------
# set trap for CNTL-C and execute main routine
# ----------------------------------------------------------------
trap cntlCtrap INT # unmount /cd and remove the cdrom ftrap
mainRoutine | tee $consoleOutputFile 2>&1

